﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;
using System.Windows.Media.Effects;
using System.Diagnostics;

namespace XamDataGridPerformance
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        int smallDatasetCount = 500;
        int largeDatasetCount = 10000;
        TickerBuffer tickerBuffer;
        ResourceDictionary hoverlessStyle = new ResourceDictionary() { Source = new Uri("HoverlessStyles.xaml", UriKind.Relative) };
        ResourceDictionary readonlyStyle = new ResourceDictionary() { Source = new Uri("ReadOnlyHighPerformanceStyle.xaml", UriKind.Relative) };
        public MainWindow()
        {
            tickerBuffer = new TickerBuffer(DataLoader.HistoricalData, smallDatasetCount);
            InitializeComponent();
            Loaded += new RoutedEventHandler(Window_Loaded);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (Debugger.IsAttached)
                MessageBox.Show(@"You're running the demo in debug mode which will overestimate CPU load values. 
Please run the demo without debugger attached to produce CPU load values you will experience in the production environment.",
                    "XamDataGrid Performance Demo",
                    MessageBoxButton.OK,
                    MessageBoxImage.Exclamation);

            xamDataGrid1.DataSource = tickerBuffer;

            cellContainerGenerationCombo.ItemsSource = Enum.GetValues(typeof(CellContainerGenerationMode));
            scrollingModeCombo.ItemsSource = Enum.GetValues(typeof(ScrollingMode));

            instructionsTb.Text = @"Please go through the following cases to see how the various settings above affect XamDataGrid performance:
1. Without changing any settings, scroll horizontally to see baseline performance
2. Bind a large amount of data - 10K, scroll vertically
3. Enable immediate scrolling mode, scroll vertically
4. Enable/Disable the drop shadow bitmap effect and scroll horizontally. Please note scrolling without the bitmap effect is faster
5. See impact of row highlighting on CPU usage:
    5.1 Move mouse across rows, highlighting them. Note CPU usage goes up when many rows are highlighted
    5.2 Change the cell style from Normal (includes hover effects and editors in edit mode), to hoverless (no hover effects, with editors in edit mode)
    5.3 Move mouse across rows, note there's no more highlighting. Note CPU usage remains low
6. Change the cell style to read-only (no hover effects, no editors in edit mode). Please note:
    6.1 No row highlighting, no editors are shown in edit mode. This style is only useful for read-only scenarios
    6.1 Scroll horizontally/vertically to see the level performance with the now much simplest cell template
6. Set cell container generation mode to Recycle, scroll horizontally. Please note:
    6.1 There's no initial loading time
    6.2 There's a choppiness in horizontal scrolling, which is always present going to the end and back through all the columns
7. Set cell container generation mode to LazyLoad, scroll horizontally Please note:
    7.1 There's no initial loading time
    7.2 There's a choppiness in horizontal scrolling only the first time columns are brought into view. Having scrolled through all columns to the right, all CellValuePresenters are cached, and subsequent horizontal scrolling is smooth
8. Set cell container generation mode to PreLoad, note amount of initial loading time, scroll horizontally
    8.1 There's a noticable initial loading time (due to the large number of columns and complex editor controls)
    8.2 Horizontal scrolling is smooth right from the start, as all CellValuePresenters have been pre-loaded
";
            
        }

        private void highDataVolumeRadio_Checked(object sender, RoutedEventArgs e)
        {
            tickerBuffer.WindowSize = largeDatasetCount;
            tickerBuffer.AddRecordsFromHistoricalData(largeDatasetCount);
        }

        private void lowDataVolumeRadio_Checked(object sender, RoutedEventArgs e)
        {
            tickerBuffer.WindowSize = smallDatasetCount;
            tickerBuffer.AddRecordsFromHistoricalData(smallDatasetCount);
        }

        private void bitmapEffectChk_Click(object sender, RoutedEventArgs e)
        {
            if ((bool)bitmapEffectChk.IsChecked)
            {
                tabControl.BitmapEffect = new DropShadowBitmapEffect();
            }
            else
            {
                tabControl.BitmapEffect = null;
            }
        }

        private void readonlyStyleRadio_Checked(object sender, RoutedEventArgs e)
        {
            if (xamDataGrid1 == null)
                return;

            xamDataGrid1.Resources.MergedDictionaries.Clear();
            xamDataGrid1.Resources.MergedDictionaries.Add(readonlyStyle);
            //forcing the reuse of cell elements, now that they all the an identical template (textblock), and no editor is rendered
            xamDataGrid1.FieldSettings.ForceCellVirtualization = true;
            //disabling edit, since there are no editors anyway
            xamDataGrid1.FieldSettings.AllowEdit = false;
        }

        private void hoverlessStyleRadio_Checked(object sender, RoutedEventArgs e)
        {
            if (xamDataGrid1 == null)
                return;

            xamDataGrid1.Resources.MergedDictionaries.Clear();
            xamDataGrid1.Resources.MergedDictionaries.Add(hoverlessStyle);
            xamDataGrid1.FieldSettings.ForceCellVirtualization = false;
            xamDataGrid1.FieldSettings.AllowEdit = true;
        }

        private void normalStyleRadio_Checked(object sender, RoutedEventArgs e)
        {
            if (xamDataGrid1 == null)
                return;

            xamDataGrid1.Resources.Clear();
            xamDataGrid1.Resources.MergedDictionaries.Clear();
            xamDataGrid1.FieldSettings.ForceCellVirtualization = false;
            xamDataGrid1.FieldSettings.AllowEdit = true;
        }
    }
}
